#include <gtk/gtk.h>
#include <stdlib.h>
#include <math.h>
#include <hildon-widgets/hildon-note.h>
#include <curl/multi.h>
#include <errno.h>
#include <glib/gstdio.h>

#include "map_download.h"
#include "map_widget.h"
#include "ui.h"


const gchar * MAP_DOWNLOADER_FW_DBUS_SERVICE = "org.cityguide.maptileloader";
const gchar * MAP_DOWNLOADER_FW_DBUS_OBJECT_PATH = "/org/cityguide/maptileloader";
const gchar * MAP_DOWNLOADER_FW_DBUS_INTERFACE = "org.cityguide.maptileloader";
const gchar * MAP_DOWNLOADER_FW_FUNCTION_MAPTILE_REQUEST = "MaptileLoaderGetMaptile";
const gchar * MAP_DOWNLOADER_FW_FUNCTION_GET_SUPPORTED_MAPS = "MaptileLoaderGetSupportedMaps";
const gchar * MAP_DOWNLOADER_FW_FUNCTION_SET_REPOFILE = "MaptileLoaderSetRepofile";

#define TILE_SIZE_PIXELS (256)
#define TILE_SIZE_P2 (8)
#define BUFFER_SIZE (2048)
#define PI   (3.14159265358979323846f)
#define WORLD_SIZE_UNITS (2 << (MAX_ZOOM + TILE_SIZE_P2))
#define MERCATOR_SPAN (-6.28318377773622f)
#define MERCATOR_TOP (3.14159188886811f)

#define latlon2unit(lat, lon, unitx, unity) { \
    gfloat tmp; \
    unitx = (lon + 180.f) * (WORLD_SIZE_UNITS / 360.f) + 0.5f; \
    tmp = sinf(lat * (PI / 180.f)); \
    unity = 0.5f + (WORLD_SIZE_UNITS / MERCATOR_SPAN) \
        * (logf((1.f + tmp) / (1.f - tmp)) * 0.5f - MERCATOR_TOP); \
}
static guint _zoom = 3;
#define MAX_ZOOM 16


static gchar *
map_widget_get_maptile(Adata *adata, guint tilex, guint tiley, guint zoom, const  gchar *maptype)
{
    int ret = 0;
    gchar *maptile = NULL;
    osso_rpc_t retval;

        maptile = g_strdup(retval.value.s);
//        g_debug("Received maptile location %s from %s service\n", maptile, MAP_WIDGET_FW_DBUS_SERVICE);

        osso_rpc_set_timeout(adata->osso, 9000);

        ret = osso_rpc_run( adata->osso,
                            MAP_DOWNLOADER_FW_DBUS_SERVICE,
                            MAP_DOWNLOADER_FW_DBUS_OBJECT_PATH,
                            MAP_DOWNLOADER_FW_DBUS_INTERFACE,
                            MAP_DOWNLOADER_FW_FUNCTION_MAPTILE_REQUEST,
                            &retval,
                            DBUS_TYPE_STRING, maptype,
                            DBUS_TYPE_UINT32, tilex,
                            DBUS_TYPE_UINT32, tiley,
                            DBUS_TYPE_UINT32, zoom,
                            DBUS_TYPE_INVALID );
    
        if ( ret != OSSO_OK )
            g_warning("Could not connect to %s service!\n", MAP_DOWNLOADER_FW_DBUS_SERVICE);
        else
        {
            maptile = g_strdup(retval.value.s);
//            g_debug("Received maptile location %s from %s service\n", maptile, MAP_WIDGET_FW_DBUS_SERVICE);
        }
        osso_rpc_free_val(&retval);
    

    return maptile;
}




gboolean mapman_by_area(gfloat start_lat, 
			gfloat start_lon,
		        gfloat end_lat, 
			gfloat end_lon, 
			Adata* adata, const gchar* maptype,gboolean zoom_levels[17])
{

    printf(" f area %f/%f to %f/%f \n",start_lat, start_lon, end_lat, end_lon);

    guint start_unitx, start_unity, end_unitx, end_unity;
    guint num_maps = 0;
    guint i;
    gchar *buffer;
    GtkWidget *confirm;
    gchar *downl_map;
    latlon2unit(start_lat, start_lon, start_unitx, start_unity);
    latlon2unit(end_lat, end_lon, end_unitx, end_unity);

    /* Swap if they specified flipped lats or lons. */
    if(start_unitx > end_unitx)
    {
        guint swap = start_unitx;
        start_unitx = end_unitx;
        end_unitx = swap;
    }
    if(start_unity > end_unity)
    {
        guint swap = start_unity;
        start_unity = end_unity;
        end_unity = swap;
    }

    /* First, get the number of maps to download. */
    for(i = 0; i < MAX_ZOOM; i++)
    {
            guint start_tilex, start_tiley, end_tilex, end_tiley;
            start_tilex = unit2ztile(start_unitx, i);
            start_tiley = unit2ztile(start_unity, i);
            end_tilex = unit2ztile(end_unitx, i);
            end_tiley = unit2ztile(end_unity, i);
            if (zoom_levels[i])
            {
            num_maps += (end_tilex - start_tilex + 1)
                * (end_tiley - start_tiley + 1);
            }
    }
    printf("Number of maps is %d\n",num_maps);   


    buffer = g_strdup_printf("Confirm download of %d maps\n(%.2fMB - %.2fMB)",num_maps, num_maps * 6e-3,num_maps, num_maps * 18e-3);


    
    confirm = hildon_note_new_confirmation(GTK_WINDOW(adata->window), buffer);

    if(GTK_RESPONSE_OK != gtk_dialog_run(GTK_DIALOG(confirm)))
    {
        gtk_widget_destroy(confirm);
        return FALSE;
    }
    for(i = 0; i < MAX_ZOOM; i++)
    {
       printf("kierros %d,to do %d\n",i,zoom_levels[i]);
       if (zoom_levels[i])
       {
	    guint start_tilex, start_tiley, end_tilex, end_tiley;
            guint tilex, tiley;
            start_tilex = unit2ztile(start_unitx, i);
            start_tiley = unit2ztile(start_unity, i);
            end_tilex = unit2ztile(end_unitx, i);
            end_tiley = unit2ztile(end_unity, i);
            for(tiley = start_tiley; tiley <= end_tiley; tiley++)
              {  
		for(tilex = start_tilex; tilex <= end_tilex; tilex++)
                 {  
		 
                 downl_map = map_widget_get_maptile(adata, tilex, tiley,i,maptype);
                 
                 printf("Map_downloaded: %s\n", downl_map);
		 //printf("tilex: %d, tiley: %d, i: %d\n",tilex, tiley, i);
      		 }
              }
       }
    }		
    gtk_widget_destroy(confirm);
    return TRUE;
}

